#include <cstdio>
#include <iostream> // cout...
#include <fstream> // ofstream
#include <sstream> //stringstream
#include <cstdlib> //needed for vector too!!
#include <cstring>
#include <cmath>
#include <algorithm> // required for min_element with ubuntu linux g++...
#include <vector>
//#include <ctime>
//#include <limits>

#include "lattice_s.h"
#include "settings.h"
#include "TimeVarPars.h"


using namespace std;

string boundaryType="Absorbing";

bool warningBool = false;

void replaceAll(std::string& str, const std::string& from, const std::string& to) {
    size_t start_pos = 0;
    while((start_pos = str.find(from, start_pos)) != std::string::npos) {
        str.replace(start_pos, from.length(), to);
        start_pos += to.length(); // In case 'to' contains 'from', like replacing 'x' with 'yx'
    }
}


int cmp_nocase_no__(const string& cs, const string& cs2) {

    string s(cs); // makes local copy that one could modify. Aletrnatively, (string& s, string& s2) [no const] would modify its arguments
    string s2(cs2);
    replaceAll(s,"_","");
    replaceAll(s2,"_","");

	return(cmp_nocase(s,s2));
}


int evaluateBool(bool &boolean, string buf) { // safe assignment of value `buf' to 'boolean'
	stringstream strstr(buf);
    string locstring;
    strstr>>locstring;
    if(cmp_nocase(locstring,"")==0 || cmp_nocase(locstring,"T")==0 ||
          cmp_nocase(locstring,"True")==0 || cmp_nocase(locstring,"Yes")==0 ||
          cmp_nocase(locstring,"Y")==0)
       boolean=true;
    else if(cmp_nocase(locstring,"F")==0 || cmp_nocase(locstring,"False")==0 ||
          cmp_nocase(locstring,"No")==0 || cmp_nocase(locstring,"N")==0)
       boolean=false;
    else {
        cout<<"(!) Suspicious specification for a boolean: "<< buf <<endl;
		cout<<"(!) Only \"\", \"T\", \"True\", \"Yes\", \"Y\", \"F\", \"False\", \"No\", and \"N\" are allowed"<<endl;
		cout<<"I exit..."<< endl;
		if (cinGetOnError) cin.get();
		exit(-1);
    }
return(0);
}

//	Initializing boolean locus/marker parameter vectors from the settings file "SettingsFilename"
void evaluateBoolVector(string buf, vector<bool>& varBoolVector, string displayVar) {
    if (cmp_nocase(buf,"") == 0 || cmp_nocase(buf,"T") == 0 ||
          cmp_nocase(buf,"True") == 0 || cmp_nocase(buf,"Yes") == 0 ||
          cmp_nocase(buf,"Y") == 0)
    	varBoolVector.push_back(true);
    else if (cmp_nocase(buf,"F") == 0 || cmp_nocase(buf,"False") == 0 ||
          cmp_nocase(buf,"No") == 0 || cmp_nocase(buf,"N") == 0)
    	varBoolVector.push_back(false);
    else {
		cerr << "\nUnrecognized boolean argument(s) for " << displayVar << " in " << SettingsFilename;
		cerr << "\nOnly \"\", \"T\", \"True\", \"Yes\", \"Y\", \"F\", \"False\", \"No\", and \"N\" are allowed.\n";
		cerr << "\nAborting IBDSim...Press any key to exit.\n";
		if (cinGetOnError)
			cin.get();
		exit(-1);
    }
}

//  transforms unspecified/partly specified/marker-wise specified parameter values into a locus-wise specification
template <typename T>
void expandParamVector(vector<T> &paramVector, string dispVar) {

//	IF the parameter vector is of type "int" type cast the void pointer correspondingly.
//	The following code is the same for "double", "string" and "bool" cases below.

	size_t vectorSize = paramVector.size();

	//	IF the particular parameter isn't declared in the settings file, use the respective default value found at the zeroth index.
	if (vectorSize == 1)
		paramVector.assign(n_locus + 1, paramVector[0]);
	//	ELSE IF it contains a single value, use the same for the total number of simulated loci irrespective of their marker type.
	else if (vectorSize == 2)
		paramVector.assign(n_locus + 1, paramVector[1]);
	//	ELSE IF the parameter values have been declared marker-wise, use them for all loci of the respective marker.
	else if ((vectorSize - 1) == (unsigned int)nMarker)
		for (int i = vectorSize; i >= 2; i--)
			paramVector.insert((paramVector.begin() + i), locusNumVector[i - 1] - 1, paramVector[i - 1]);
	//	ELSE unspecified format, report it and quit IBDSim.
	else {
		cerr << "\nWrong number of arguments for \"" << dispVar << "\". \nPlease check " << SettingsFilename
				<< endl << "Aborting IBDSim...Press any key to exit.";
		if (cinGetOnError)
			cin.get();
		exit(-1);
	}
/*
	//	FOR DEBUGGING PURPOSES ONLY: code for outputting parameter contents locus-wise
	cout << endl << dispVar << endl;
	for (vector<T>::iterator it = paramVector.begin() + 1; it < paramVector.end(); it++)
		cout << *it << " ";
	cout << endl;
 */
}


//Lecture des paramtres dans fichier
int read_settings_file(const char filename[]) {
    string buf,var;
//stringstream good mostly for string/numbers conversions
    unsigned int pos;
    int tempo;
    char bidon;
    bool hacktempo=true;

    Spec_Sample_Coord[x].resize(0);
    Spec_Sample_Coord[y].resize(0);
    ifstream settings(filename,ios::in);

    if(!settings.is_open()) {
        cerr << "Unable to open file "<<filename << " in read_settings_file();" << endl;
        cerr<<" Check settings. I exit."<<endl;
        if (cinGetOnError) cin.get();
        exit(-1);
	} else do {
		getline(settings,buf);
		if(buf.length()==0) goto nextline;
		while((buf[0]==' ')||(buf[0]=='\t')||(buf[0]=='\r')) {buf.erase(0,1);}//vire les blancs initiaux
		pos=std::min(buf.find('='),std::min(buf.find('\t'),buf.length()));
//		pos=std::min(buf.find('='),buf.length());
        if ((buf[pos])=='=') while (buf[pos-1]==' '||(buf[pos-1]=='\t')) {buf.erase(pos-1,1); pos--;}// vire les blancs et les tabulations avant le =
		var=buf.substr(0,pos).c_str();
		if(var.length()==0) goto nextline;
		if(var[0]=='/'||var[0]=='%'||var[0]=='#') goto nextline;//RL pour ne pas afficher les commentaires

#ifdef GOTO
_gotoxy(0,15);
#endif
        if (var[var.size()-1]=='0') {
            cerr<<"Automatically removing final '0' from "<<var<<endl;
            var=var.substr(0,var.size()-1);
        }
		if(cmp_nocase_no__(var,"Pause")==0) {
/* Pause determines only two correct contexts for cin.get():
       cerr<< error message + if(cinGetOnError) cin.get() + exit
and
       cout<< some info + if(pauseGP) cin.get() + execution continues
cinGetOnError is true at declaration in genepop.cpp but may then be set to false in latin.cpp
*/
			string locstring;
			stringstream strstr(buf.substr(pos+1));
			strstr>>locstring;
			if(cmp_nocase_no__(buf.substr(pos+1),"Final")==0) pauseGP=true;
			if(cmp_nocase_no__(locstring,"OnError")==0) cinGetOnError=true;
			if((cmp_nocase_no__(locstring,"Default")==0)) {cinGetOnError=false; pauseGP=false;}
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Data_File_Name")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>fichier_genepop;
			goto nextline;
		}
		if(cmp_nocase_no__(var,".txt_extension")==0) {
			evaluateBool(txt_extensionbool,buf.substr(pos+1));
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Genepopfile_extension")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>Genepopfile_extension;
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Run_Number")==0 || cmp_nocase_no__(var,"Dataset_Number")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>repet;
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Random_Seeds")==0 || cmp_nocase_no__(var,"Random_Seed")==0) {
			string reste=buf.substr(pos+1);
			stringstream strstr(reste);
			//int i=0;
			while (!strstr.eof()) {
				strstr>>tempo;
				/*strstr.get(); to skip any one-character separator which is necessarily here
				is not sufficient is there are whitespaces after the last value. If so, eof is not reached
				but no new value is read -> the last value is duplicated */
                 while (!strstr.eof() && !(isdigit(bidon=strstr.peek())) && bidon!='.') strstr.get();
                 Seeds=tempo;
                 //i++;
			}
			strstr.clear(); // c'est le truc essentiel pour le reutiliser... snif
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Locus_Number") == 0) {
			buf = buf.substr(pos + 1);
			string arg;
			pos = 0;
            while (buf.length() > 0) {
			   while (buf.length() > pos && !isdigit(buf[pos])) {
				   if ((buf[pos] != ' ') && (buf[pos] != ',') && (buf[pos] != '\t')) {
					   cerr << "Unidentified specification \"" << buf[pos] << "\" for \"" << var << "\" in "
					   	    << filename << "! Ignored." << endl;
					   if (!warningBool)
						   warningBool = true;
				   }
				   pos++;
			   }
			   if (buf.length() == pos)
				   goto nextline;
			   buf = buf.substr(pos);
			   pos = 0;
			   while (buf.length() > pos && isdigit(buf[pos]))
				   pos++;
			   arg = buf.substr(0,pos);
			   locusNumVector.push_back(atoi(arg.c_str()));
               if (buf.length() > pos) {
    				buf = buf.substr(pos+1);
    			   	pos = 0;
			   } else goto nextline;
			}
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Mutation_Rate") == 0) {
			buf = buf.substr(pos + 1);
			pos = 0;
            while (buf.length() > 0) {
			   while (buf.length() > pos && !(buf[pos] == '.') && !isdigit(buf[pos])) {
				   if ((buf[pos] != ' ') && (buf[pos] != ',') && (buf[pos] != '\t')) {
					   cerr << "Unidentified specification \"" << buf[pos] << "\" for \"" << var << "\" in "
					   	    << filename << "! Ignored." << endl;
					   if (!warningBool)
						   warningBool = true;
				   }
				   pos++;
			   }
			   if (buf.length() == pos)
				   goto nextline;
			   buf = buf.substr(pos);
			   pos = 0;
			   while (buf.length() > pos && (buf[pos] == '.' || isdigit(buf[pos])))
				   pos++;
			   string strstr(buf.substr(0,pos));
			   double stringToFloat = atof(strstr.c_str());
			   mutRateVector.push_back(stringToFloat);
               if (buf.length() > pos) {
    				buf = buf.substr(pos+1);
    			   	pos = 0;
			   } else goto nextline;
			}
			goto nextline;
		}
		if (cmp_nocase_no__(var,"Mutation_Model") == 0) {
			buf = buf.substr(pos + 1);
			string arg;
			pos = 0;
            while (buf.length() > 0) {
			   while (buf.length() > pos && !isalnum(buf[pos])) {
				   if ((buf[pos] != ' ') && (buf[pos] != ',') && (buf[pos] != '\t')) {
					   cerr << "Unidentified specification \"" << buf[pos] << "\" for \"" << var << "\" in "
					   	    << filename << "! Ignored." << endl;
					   if (!warningBool)
						   warningBool = true;
				   }
				   pos++;
			   }
			   if (buf.length() == pos)
				   goto nextline;
			   buf = buf.substr(pos);
			   pos = 0;
			   while (buf.length() > pos && isalnum(buf[pos]))
				   pos++;
			   arg = buf.substr(0,pos);
			   mutModelIDVector.push_back(arg);
			   if (!((cmp_nocase_no__(mutModelIDVector.back(),"SMM") == 0)
				   || (cmp_nocase_no__(mutModelIDVector.back(),"IAM") == 0)
				   || (cmp_nocase_no__(mutModelIDVector.back(),"KAM") == 0)
				   || (cmp_nocase_no__(mutModelIDVector.back(),"TPM") == 0)
				   || (cmp_nocase_no__(mutModelIDVector.back(),"GSM") == 0)
				   || (cmp_nocase_no__(mutModelIDVector.back(),"JC69") == 0)
				   || (cmp_nocase_no__(mutModelIDVector.back(),"K80") == 0)
				   || (cmp_nocase_no__(mutModelIDVector.back(),"F81") == 0)
				   || (cmp_nocase_no__(mutModelIDVector.back(),"HKY85") == 0)
				   || (cmp_nocase_no__(mutModelIDVector.back(),"TN93") == 0)
				   || (cmp_nocase_no__(mutModelIDVector.back(),"SNP") == 0)
				   || (cmp_nocase_no__(mutModelIDVector.back(),"ISM") == 0))) {
				   cerr << "\nUnrecognized Mutation_Model argument(s) in " << filename;
				   cerr << "\nCurrently only a combination of SMM, IAM, KAM, TPM, GSM, JC69, K80, "
						   "F81, HKY85, TN93, SNP or ISM models are allowed.\n";
				   cerr << "\nAborting IBDSim...Press any key to exit.\n";
				   if (cinGetOnError)
					   cin.get();
				   exit(-1);
			   }
			   if (cmp_nocase_no__(mutModelIDVector.back(),"ISM") == 0
					|| cmp_nocase_no__(mutModelIDVector.back(),"SNP") == 0
					|| cmp_nocase_no__(mutModelIDVector.back(),"JC69") == 0
					|| cmp_nocase_no__(mutModelIDVector.back(),"K80") == 0
					|| cmp_nocase_no__(mutModelIDVector.back(),"F81") == 0
					|| cmp_nocase_no__(mutModelIDVector.back(),"HKY85") == 0
					|| cmp_nocase_no__(mutModelIDVector.back(),"TN93") == 0)
				   specifyLocusSeq = true;
               if (buf.length() > pos) {
    				buf = buf.substr(pos+1);
    			   	pos = 0;
			   } else goto nextline;
			}
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Min_Allele_Number") == 0) { //FR->RL virer ?????
			buf = buf.substr(pos + 1);
			string arg;
			pos = 0;
            while (buf.length() > 0) {
			   while (buf.length() > pos && !isdigit(buf[pos])) {
				   if ((buf[pos] != ' ') && (buf[pos] != ',') && (buf[pos] != '\t')) {
					   cerr << "Unidentified specification \"" << buf[pos] << "\" for \"" << var << "\" in "
					   	    << filename << "! Ignored." << endl;
					   if (!warningBool)
						   warningBool = true;
				   }
				   pos++;
			   }
			   if (buf.length() == pos)
				   goto nextline;
			   buf = buf.substr(pos);
			   pos = 0;
			   while (buf.length() > pos && isdigit(buf[pos]))
				   pos++;
			   arg = buf.substr(0,pos);
			   minAlleleVector.push_back(atoi(arg.c_str()));
               if (buf.length() > pos) {
    				buf = buf.substr(pos+1);
    			   	pos = 0;
			   } else goto nextline;
			}
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Polymorphic_Loci_only") == 0 || cmp_nocase_no__(var,"Polymorphic_Locus_only") == 0
				|| cmp_nocase_no__(var,"Polymorphic_Loci") == 0 || cmp_nocase_no__(var,"Polymorphic_Locus") == 0) {
			buf = buf.substr(pos + 1);
			string arg;
			pos = 0;
            while (buf.length() > 0) {
			   while (buf.length() > pos && !isalpha(buf[pos])) {
				   if ((buf[pos] != ' ') && (buf[pos] != ',') && (buf[pos] != '\t')) {
					   cerr << "Unidentified specification \"" << buf[pos] << "\" for \"" << var << "\" in "
					   	    << filename << "! Ignored." << endl;
					   if (!warningBool)
						   warningBool = true;
				   }
				   pos++;
			   }
			   if (buf.length() == pos)
				   goto nextline;
			   buf = buf.substr(pos);
			   pos = 0;
			   while (buf.length() > pos && isalpha(buf[pos]))
				   pos++;
			   arg = buf.substr(0,pos);
			   evaluateBoolVector(arg, polyLociBoolVector, var);
               if (buf.length() > pos) {
    				buf = buf.substr(pos+1);
    			   	pos = 0;
			   } else goto nextline;
			}
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Minor_Allele_Frequency") == 0) {
			buf = buf.substr(pos + 1);
			pos = 0;
            while (buf.length() > 0) {
			   while (buf.length() > pos && !(buf[pos] == '.') && !isdigit(buf[pos])) {
				   if ((buf[pos] != ' ') && (buf[pos] != ',') && (buf[pos] != '\t')) {
					   cerr << "Unidentified specification \"" << buf[pos] << "\" for \"" << var << "\" in "
					   	    << filename << "! Ignored." << endl;
					   if (!warningBool)
						   warningBool = true;
				   }
				   pos++;
			   }
			   if (buf.length() == pos)
				   goto nextline;
			   buf = buf.substr(pos);
			   pos = 0;
			   while (buf.length() > pos && (buf[pos] == '.' || isdigit(buf[pos])))
				   pos++;
			   string strstr(buf.substr(0,pos));
			   double stringToFloat = atof(strstr.c_str());
			   minorAlleleFreqVector.push_back(stringToFloat);
               if (buf.length() > pos) {
    				buf = buf.substr(pos+1);
    			   	pos = 0;
			   } else goto nextline;
			}
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Variable_Mutation_Rate") == 0) {
			buf = buf.substr(pos + 1);
			string arg;
			pos = 0;
            while (buf.length() > 0) {
			   while (buf.length() > pos && !isalpha(buf[pos])) {
				   if ((buf[pos] != ' ') && (buf[pos] != ',') && (buf[pos] != '\t')) {
					   cerr << "Unidentified specification \"" << buf[pos] << "\" for \"" << var << "\" in "
					   	    << filename << "! Ignored." << endl;
					   if (!warningBool)
						   warningBool = true;
				   }
				   pos++;
			   }
			   if (buf.length() == pos)
				   goto nextline;
			   buf = buf.substr(pos);
			   pos = 0;
			   while (buf.length() > pos && isalpha(buf[pos]))
				   pos++;
			   arg = buf.substr(0,pos);
			   evaluateBoolVector(arg, varMutRateBoolVector, var);
               if (buf.length() > pos) {
    				buf = buf.substr(pos+1);
    			   	pos = 0;
			   } else goto nextline;
			}
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Allelic_Lower_Bound") == 0) {
			buf = buf.substr(pos + 1);
			string arg;
			pos = 0;
            while (buf.length() > 0) {
			   while (buf.length() > pos && !isalnum(buf[pos])) {
				   if ((buf[pos] != ' ') && (buf[pos] != ',') && (buf[pos] != '\t')) {
					   cerr << "Unidentified specification \"" << buf[pos] << "\" for \"" << var << "\" in "
					   	    << filename << "! Ignored." << endl;
					   if (!warningBool)
						   warningBool = true;
				   }
				   pos++;
			   }
			   if (buf.length() == pos)
				   goto nextline;
			   buf = buf.substr(pos);
			   pos = 0;
			   while (buf.length() > pos && isalnum(buf[pos]))
				   pos++;
			   arg = buf.substr(0,pos);
			   if (cmp_nocase_no__(arg,"NA") == 0)
				   arg = "-1";
			   kMinVector.push_back(atoi(arg.c_str()));
               if (buf.length() > pos) {
    				buf = buf.substr(pos+1);
    			   	pos = 0;
			   } else goto nextline;
			}
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Allelic_Upper_Bound") == 0) {
			buf = buf.substr(pos + 1);
			string arg;
			pos = 0;
            while (buf.length() > 0) {
			   while (buf.length() > pos && !isalnum(buf[pos])) {
				   if ((buf[pos] != ' ') && (buf[pos] != ',') && (buf[pos] != '\t')) {
					   cerr << "Unidentified specification \"" << buf[pos] << "\" for \"" << var << "\" in "
					   	    << filename << "! Ignored." << endl;
					   if (!warningBool)
						   warningBool = true;
				   }
				   pos++;
			   }
			   if (buf.length() == pos)
				   goto nextline;
			   buf = buf.substr(pos);
			   pos = 0;
			   while (buf.length() > pos && isalnum(buf[pos]))
				   pos++;
			   arg = buf.substr(0,pos);
			   if (cmp_nocase_no__(arg,"NA") == 0)
				   arg = "-1";
			   kMaxVector.push_back(atoi(arg.c_str()));
               if (buf.length() > pos) {
    				buf = buf.substr(pos+1);
    			   	pos = 0;
			   } else goto nextline;
			}
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Allelic_State_MRCA") == 0) {
			buf = buf.substr(pos + 1);
			string arg;
			pos = 0;
            while (buf.length() > 0) {
			   while (buf.length() > pos && !isalnum(buf[pos])) {
				   if ((buf[pos] != ' ') && (buf[pos] != ',') && (buf[pos] != '\t')) {
					   cerr << "Unidentified specification \"" << buf[pos] << "\" for \"" << var << "\" in "
					   	    << filename << "! Ignored." << endl;
					   if (!warningBool)
						   warningBool = true;
				   }
				   pos++;
			   }
			   if (buf.length() == pos)
				   goto nextline;
			   buf = buf.substr(pos);
			   pos = 0;
			   while (buf.length() > pos && isalnum(buf[pos]))
				   pos++;
			   arg = buf.substr(0,pos);
			   if (cmp_nocase_no__(arg,"NA") == 0)
				   arg = "-1";
			   kIniVector.push_back(atoi(arg.c_str()));
               if (buf.length() > pos) {
    				buf = buf.substr(pos+1);
    			   	pos = 0;
			   } else goto nextline;
			}
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Repeated_motif_size") == 0) {
			buf = buf.substr(pos + 1);
			string arg;
			pos = 0;
            while (buf.length() > 0) {
			   while (buf.length() > pos && !isalnum(buf[pos])) {
				   if ((buf[pos] != ' ') && (buf[pos] != ',') && (buf[pos] != '\t')) {
					   cerr << "Unidentified specification \"" << buf[pos] << "\" for \"" << var << "\" in "
					   	    << filename << "! Ignored." << endl;
					   if (!warningBool)
						   warningBool = true;
				   }
				   pos++;
			   }
			   if (buf.length() == pos)
				   goto nextline;
			   buf = buf.substr(pos);
			   pos = 0;
			   while (buf.length() > pos && isalnum(buf[pos]))
				   pos++;
			   arg = buf.substr(0,pos);
			   if (cmp_nocase_no__(arg,"NA") == 0)
				   arg = "-1";
			   motifSizeVector.push_back(atoi(arg.c_str()));
               if (buf.length() > pos) {
    				buf = buf.substr(pos+1);
    			   	pos = 0;
			   } else goto nextline;
			}
			goto nextline;
		}
		if(cmp_nocase_no__(var,"SMM_Probability_In_TPM") == 0) {
			buf = buf.substr(pos + 1);
			pos = 0;
            while (buf.length() > 0) {
			   while (buf.length() > pos && !(buf[pos] == '.') && !isalnum(buf[pos])) {
				   if ((buf[pos] != ' ') && (buf[pos] != ',') && (buf[pos] != '\t')) {
					   cerr << "Unidentified specification \"" << buf[pos] << "\" for \"" << var << "\" in "
					   	    << filename << "! Ignored." << endl;
					   if (!warningBool)
						   warningBool = true;
				   }
				   pos++;
			   }
			   if (buf.length() == pos)
				   goto nextline;
			   buf = buf.substr(pos);
			   pos = 0;
			   while (buf.length() > pos && (buf[pos] == '.' || isalnum(buf[pos])))
				   pos++;
			   string strstr(buf.substr(0,pos));
			   if (cmp_nocase_no__(strstr,"NA") == 0)
				   strstr = "-1";
			   double stringToFloat = atof(strstr.c_str());
			   pSMMVector.push_back(stringToFloat);
               if (buf.length() > pos) {
    				buf = buf.substr(pos+1);
    			   	pos = 0;
			   } else goto nextline;
			}
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Geometric_Variance_In_TPM") == 0) {
			buf = buf.substr(pos + 1);
			pos = 0;
            while (buf.length() > 0) {
			   while (buf.length() > pos && !(buf[pos] == '.') && !isalnum(buf[pos])) {
				   if ((buf[pos] != ' ') && (buf[pos] != ',') && (buf[pos] != '\t')) {
					   cerr << "Unidentified specification \"" << buf[pos] << "\" for \"" << var << "\" in "
					   	    << filename << "! Ignored." << endl;
					   if (!warningBool)
						   warningBool = true;
				   }
				   pos++;
			   }
			   if (buf.length() == pos)
				   goto nextline;
			   buf = buf.substr(pos);
			   pos = 0;
			   while (buf.length() > pos && (buf[pos] == '.' || isalnum(buf[pos])))
				   pos++;
			   string strstr(buf.substr(0,pos));
			   if (cmp_nocase_no__(strstr,"NA") == 0)
				   strstr = "-1";
			   double stringToFloat = atof(strstr.c_str());
			   geomTPMVector.push_back(stringToFloat);
               if (buf.length() > pos) {
    				buf = buf.substr(pos+1);
    			   	pos = 0;
			   } else goto nextline;
			}
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Geometric_Variance_In_GSM") == 0) {
			buf = buf.substr(pos + 1);
			pos = 0;
            while (buf.length() > 0) {
			   while (buf.length() > pos && !(buf[pos] == '.') && !isalnum(buf[pos])) {
				   if ((buf[pos] != ' ') && (buf[pos] != ',') && (buf[pos] != '\t')) {
					   cerr << "Unidentified specification \"" << buf[pos] << "\" for \"" << var << "\" in "
					   	    << filename << "! Ignored." << endl;
					   if (!warningBool)
						   warningBool = true;
				   }
				   pos++;
			   }
			   if (buf.length() == pos)
				   goto nextline;
			   buf = buf.substr(pos);
			   pos = 0;
			   while (buf.length() > pos && (buf[pos] == '.' || isalnum(buf[pos])))
				   pos++;
			   string strstr(buf.substr(0,pos));
			   if (cmp_nocase_no__(strstr,"NA") == 0)
				   strstr = "-1";
			   double stringToFloat = atof(strstr.c_str());
			   geomGSMVector.push_back(stringToFloat);
               if (buf.length() > pos) {
    				buf = buf.substr(pos+1);
    			   	pos = 0;
			   } else goto nextline;
			}
			goto nextline;
		}
		if(cmp_nocase_no__(var,"MRCA_Sequence")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>defMRCASequence;
			if (defMRCASequence.size())
				seqSize = (int) defMRCASequence.size();
            goto nextline;
		}
		if(cmp_nocase_no__(var,"Sequence_Size")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>seqSize;
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Transition_Transversion_ratio")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>ratio_TITV;
		    if (ratio_TITV < 0) {
		    	cerr << "\nThe \"Transition_Transversion_ratio\" should always be positive. \nPlease check "
					 << SettingsFilename << "\nAborting IBDSim...Press any key to exit.\n";
				if (cinGetOnError)
					cin.get();
				exit(-1);
		    }
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Transition1_Transition2_ratio")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>ratio_TITI;
		    if (ratio_TITI < 0) {
		    	cerr << "\nThe \"Transition1_Transition2_ratio\" should always be positive. \nPlease check "
					 << SettingsFilename << "\nAborting IBDSim...Press any key to exit.\n";
				if (cinGetOnError)
					cin.get();
				exit(-1);
		    }
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Equilibrium_Frequencies") == 0) {
			int arrayIndex = 0;
			buf = buf.substr(pos + 1);
			pos = 0;
			specifyVarBaseFreqs = true;
            while (buf.length() > 0) {
			   while (buf.length() > pos && !(buf[pos] == '.') && !isdigit(buf[pos])) {
				   if ((buf[pos] != ' ') && (buf[pos] != ',') && (buf[pos] != '\t')) {
					   cerr << "Unidentified specification \"" << buf[pos] << "\" for \"" << var << "\" in "
					   	    << filename << "! Ignored." << endl;
					   if (!warningBool)
						   warningBool = true;
				   }
				   pos++;
			   }
			   if (buf.length() == pos)
				   goto nextline;
			   buf = buf.substr(pos);
			   pos = 0;
			   while (buf.length() > pos && (buf[pos] == '.' || isdigit(buf[pos])))
				   pos++;
			   string strstr(buf.substr(0,pos));
			   double stringToFloat = atof(strstr.c_str());
			   if (arrayIndex == 4) {
					cerr << "\nError in the number of parameters for \"Equilibrium_Frequencies\". Please check "
						 << SettingsFilename << "\nAborting IBDSim...Press any key to exit.\n";
					if (cinGetOnError)
						cin.get();
					exit(-1);
			   }
			   varBaseFreqs[arrayIndex] = stringToFloat;
			   ++arrayIndex;
               if (buf.length() > pos) {
    				buf = buf.substr(pos+1);
    			   	pos = 0;
			   } else goto nextline;
			}
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Nexus_file_format") == 0) {
			string locstring;
			stringstream strstr(buf.substr(pos+1));
			strstr >> locstring;
			if(cmp_nocase_no__(locstring,"Haplotypes_and_Individuals") == 0)
				nexusFormat = locstring;
			else if(cmp_nocase_no__(locstring,"Haplotypes_only") == 0)
				nexusFormat = locstring;
			else if(cmp_nocase(locstring,"F")==0 || cmp_nocase(locstring,"False")==0 ||
			          cmp_nocase(locstring,"No")==0 || cmp_nocase(locstring,"N")==0)
				nexusFormat="F";
		    else {
				cerr << "\nUnrecognized argument for Haplotype_format in " << SettingsFilename;
				cerr << "\nOnly \"Haplotypes_only\", \"Haplotypes_and_Individuals\" and \"False\"are allowed.\n";
				cerr << "\nAborting IBDSim...Press any key to exit.\n";
				if (cinGetOnError)
					cin.get();
				exit(-1);
		    }
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Ploidy")==0) {
			string locstring;
			stringstream strstr(buf.substr(pos+1));
			strstr>>locstring;
			if(cmp_nocase_no__(locstring,"Diploid")==0) ploidy=2;
			else if(cmp_nocase_no__(locstring,"Haploid")==0) ploidy=1;
				else {cerr<<"\nBad Argument for Ploidy in " << SettingsFilename <<  " :"<<endl;
					  cerr<<"Only Diploid or Haploid are implemented"<<endl;
					  if(cinGetOnError) cin.get();
					  exit(-1);
				}
			goto nextline;
		}
		if(cmp_nocase_no__(var,"genepop")==0) {
			evaluateBool(genepopoui,buf.substr(pos+1));
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Migraine")==0) {
			evaluateBool(DG2002,buf.substr(pos+1)); //FR->RL obsolete...
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Migraine_AllStates")==0) {
			evaluateBool(AllStates,buf.substr(pos+1));  // colonne des 0 aux format DG2002 (obsolete)
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Migrate")==0) {
			evaluateBool(migrate_oui,buf.substr(pos+1));
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Migrate_Letter")==0 || cmp_nocase_no__(var,"Migrate_Lettre")==0) {
			evaluateBool(migrate_lettre_oui,buf.substr(pos+1));
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Generic_Computations")==0) {
			evaluateBool(calculoui,buf.substr(pos+1));  //FR->RL remove or rename
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Hexp")==0 || cmp_nocase_no__(var,"Hexp_Nei")==0) {
			evaluateBool(HexpNeioui,buf.substr(pos+1));
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Seq_stats")==0) {
			evaluateBool(seqStatsOui,buf.substr(pos+1));
			goto nextline;
		}
		if(cmp_nocase_no__(var,"DeltaH")==0) {
            cerr<<"DeltaH is obsolete..."<<endl;
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Allelic_Variance")==0) { // calcul variance...
			evaluateBool(Varoui,buf.substr(pos+1));
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Iterative_Identity_Probability")==0) {
			evaluateBool(suiviQ,buf.substr(pos+1));
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Iterative_Statistics")==0) {
			evaluateBool(iterativeStats,buf.substr(pos+1));
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Allelic_Number_Folder")==0) { // FR->RL virer et mettre la doc a jour
			string locstring;
			stringstream strstr(buf.substr(pos+1));
			strstr>>locstring;

			if(cmp_nocase(locstring,"true")==0) {
                cout<<"\nAllelic_Number_Folder is no more implemented :"<<endl;
				cout<<"Contact R. leblois if you really need this option"<<endl;
				getchar();
				exit(-1);
                //dossier_nbre_allele=true;
            } else if(cmp_nocase(locstring,"false")==0) dossier_nbre_allele=false;
				else {cout<<"\nBad Argument for Allelic_Number_Folder in " << SettingsFilename <<  " :"<<endl;
					  cout<<"Only true or false are possible options"<<endl;
				 	  getchar();
					  exit(-1);
				}
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Old_Version_Computation")==0) { // FR->RL virer et mettre la doc a jour
			string locstring;
			stringstream strstr(buf.substr(pos+1));
			strstr>>locstring;
			if(cmp_nocase(locstring,"true")==0) calcul_anc=true;
			else if(cmp_nocase(locstring,"false")==0) calcul_anc=false;
				else {cout<<"\nBad Argument for Old_Version_Computation in " << SettingsFilename <<  " :"<<endl;
					  cout<<"Only true or false are possible options"<<endl;
				 	  getchar();
					  exit(-1);
				}
			goto nextline;
		}

		if(cmp_nocase_no__(var,"DiagnosticTables")==0) { // FR->RL new mechanism to pass options to R in a more economical way that all the booleans in Roptions
			buf=buf.substr(pos+1);
			string arg;
			pos=0;
            while (buf.length()>0) {
			   while (buf.length()>pos && !(buf[pos]=='_') && !isalnum(buf[pos])) pos++;
			   if (buf.length()==pos) goto nextline;
			   buf=buf.substr(pos);
			   pos=0;
			   while (buf.length()>pos && (buf[pos]=='_' || isalnum(buf[pos]))) pos++;
			   arg=buf.substr(0,pos);
			   if (cmp_nocase_no__(arg,"Effective_Dispersal")==0 || cmp_nocase_no__(arg,"Realized_Dispersal")==0) {effective_disp=true;} //
			   else if (cmp_nocase_no__(arg,"Prob_Id_Matrix")==0) {Prob_Id_Matrix=true;} //
			   else if (cmp_nocase_no__(arg,"Iterative_Statistics")==0) {iterativeStats=true;} //
			   else if (cmp_nocase_no__(arg,"Iterative_Identity_Probability")==0) {suiviQ=true;} //
			   else if (cmp_nocase_no__(arg,"Allelic_Variance")==0) {Varoui=true;} //
			   else if (cmp_nocase_no__(arg,"Hexp")==0 || cmp_nocase_no__(arg,"Hexp_Nei")==0) {HexpNeioui=true;} //
			   else if (cmp_nocase_no__(arg,"Seq_stats")==0) {seqStatsOui=true;} //
			   else if (cmp_nocase_no__(arg,"Fis")==0) {Fisoui=true;} //

			   else {
                   cerr<<"Unknown DiagnosticTables arguments'"<<arg<<"'. Check settings. I exit."<<endl;
                   if (cinGetOnError) cin.get();
                   exit(-1);
                }
               if (buf.length()>pos) {
    				buf=buf.substr(pos+1);
    			   	pos=0;
			   } else goto nextline;
			}
			goto nextline;
		}
		
		if(cmp_nocase_no__(var,"Prob_Id_Matrix")==0) {
			evaluateBool(Prob_Id_Matrix,buf.substr(pos+1));
			goto nextline;
		}

		if(cmp_nocase_no__(var,"Effective_Dispersal")==0 || cmp_nocase_no__(var,"Realized_Dispersal")==0) {
			evaluateBool(effective_disp,buf.substr(pos+1));
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Constant_Dispersal")==0) {
			evaluateBool(const_disp,buf.substr(pos+1));
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Immigration_Control")==0 || cmp_nocase_no__(var,"2D_Dispersal")==0 || cmp_nocase_no__(var,"2D_Disp")==0) {
		    //FR->FR FR->RL trouver autre nom car option a un sens en 1D
			string locstring;
			stringstream strstr(buf.substr(pos+1));
			strstr>>locstring;
			if(cmp_nocase_no__(locstring,"Simple1DProduct")==0) twoD_disp="Simple1DProduct";
			else if(cmp_nocase_no__(locstring,"1DProductWithoutm0")==0) twoD_disp="1DProductWithoutm0";
			//else if(cmp_nocase(locstring,"X+YdistProb")==0) twoD_disp="X+YDistProb";
			else {cerr<<"\nBad Argument for 2D_Dispersal in " << SettingsFilename <<  " :"<<endl;
				cerr<<"Only Simple1DProduct and 1DProductWithoutm0 are possible options. I exit."<<endl;
			    if(cinGetOnError) cin.get();
				exit(-1);
			}
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Total_Range_Dispersal")==0) {
			evaluateBool(compareMath,buf.substr(pos+1));
			/** if true: dx_max est contraint par la fonction de dispersion
			 else dx_max et la dim max du reseau**/
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Lattice_Boundaries")==0 || cmp_nocase_no__(var,"Edge_Effect")==0 || cmp_nocase_no__(var,"Edge_Effects")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>boundaryType;
            goto nextline;
		}
		if(cmp_nocase_no__(var,"Max_Lattice_SizeX")==0) {
            cerr<<"Max_Lattice_SizeX is obsolete..."<<endl;
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Max_Lattice_SizeY")==0) {
            cerr<<"Max_Lattice_SizeY is obsolete..."<<endl;
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Sample_SizeX")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>dim_sample1;
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Sample_SizeY")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>dim_sample2;
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Min_Sample_CoordinateX")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>xmin_sample;
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Min_Sample_CoordinateY")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>ymin_sample;
			goto nextline;
		}
		if(cmp_nocase_no__(var,"SpecificSampleDesign_SampleSize")==0) {
            cerr<<"SpecificSampleDesign_SampleSize is obsolete..."<<endl;
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Specific_Sample_Design")==0) {
            cerr<<"Specific_Sample_Design is obsolete..."<<endl;
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Sample_Coordinates_X")==0) {
            string reste=buf.substr(pos+1);
			stringstream strstr(reste);
			Spec_Sample_Coord[x].resize(0);
			while (!strstr.eof()) {
					strstr>>tempo;
					/*strstr.get(); to skip any one-character separator which is necessarily here
					is not sufficient is there are whitespaces after the last value. If so, eof is not reached
					but no new value is read -> the last value is duplicated */
	                 while (!strstr.eof() && !(isdigit(bidon=strstr.peek())) && bidon!='.') strstr.get();
	                 Spec_Sample_Coord[x].push_back(tempo);
			}
			vector<int>::iterator it=min_element(Spec_Sample_Coord[x].begin(), Spec_Sample_Coord[x].end());
			xmin_sample=(*it);
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Sample_Coordinates_Y")==0) {
				string reste=buf.substr(pos+1);
				stringstream strstr(reste);
				Spec_Sample_Coord[y].resize(0);
				while (!strstr.eof()) {
					strstr>>tempo;
					/*strstr.get(); to skip any one-character separator which is necessarily here
					is not sufficient is there are whitespaces after the last value. If so, eof is not reached
					but no new value is read -> the last value is duplicated */
	                 while (!strstr.eof() && !(isdigit(bidon=strstr.peek())) && bidon!='.') strstr.get();
	                 Spec_Sample_Coord[y].push_back(tempo);
				}
				vector<int>::iterator it=min_element(Spec_Sample_Coord[y].begin(), Spec_Sample_Coord[y].end());
				ymin_sample=(*it);
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Ind_Per_Pop_Sampled")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>dens_sample;
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Void_Sample_Node")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>vide_sample;
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Min_Zone_CoordinateX")==0) { //FR->RL on laisse sans default pour l'instant jusqua Specific_Density_Design
			stringstream strstr(buf.substr(pos+1));
			strstr>>TVpars.back().xmin_zone;
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Max_Zone_CoordinateX")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>TVpars.back().xmax_zone;
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Min_Zone_CoordinateY")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>TVpars.back().ymin_zone;
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Max_Zone_CoordinateY")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>TVpars.back().ymax_zone;
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Specific_Density_Design")==0) {
			evaluateBool(Specific_Density_Designbool,buf.substr(pos+1));
            //remplissage du tableau densite plus loin car besoiun de DimResX0 et Y0
			goto nextline;
		}
		if(cmp_nocase_no__(var,"NewDemographicPhaseAt")==0) {
			unsigned long int locMinGen;
			CTimeVaryingParams locCTP;
			stringstream strstr(buf.substr(pos+1));
			strstr>>locMinGen;
			if (locMinGen>1) {
			    if (locMinGen<=TVpars.back().minGen) {
                    cerr<<"New demographic phase must begin at later time than previous one." <<endl;
                    cerr<<"Check use of NewDemographicPhaseAt setting. I exit." <<endl;
                    if(cinGetOnError) cin.get();
                    exit(-1);
                }
                TVpars.push_back(CTimeVaryingParams(&(TVpars.back())));
                TVpars.back().minGen=locMinGen;
			}
            //remplissage du tableau densite plus loin car besoiun de DimResX0 et Y0
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Ind_Per_Pop")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>TVpars.back().initialDens;
			TVpars.back().dens=TVpars.back().initialDens;
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Lattice_SizeX")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>TVpars.back().dimRes1;
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Lattice_SizeY")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>TVpars.back().dimRes2;
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Void_Nodes")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>TVpars.back().vide;
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Zone")==0) {
            evaluateBool(TVpars.back().zone,buf.substr(pos+1));
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Void_Nodes_Zone")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>TVpars.back().vide_zone;
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Ind_Per_Pop_Zone")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>TVpars.back().dens_zone;
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Barrier")==0) {
            evaluateBool(TVpars.back().barrier,buf.substr(pos+1));
			goto nextline;
		}
		if(cmp_nocase_no__(var,"X1_Barrier")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>TVpars.back().x1_barrier;
			goto nextline;
		}
		if(cmp_nocase_no__(var,"X2_Barrier")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>TVpars.back().x2_barrier;
			goto nextline;
		}if(cmp_nocase_no__(var,"Y1_Barrier")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>TVpars.back().y1_barrier;
			goto nextline;
		}if(cmp_nocase_no__(var,"Y2_Barrier")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>TVpars.back().y2_barrier;
			goto nextline;
		}
		if(cmp_nocase_no__(var,"BarrierCrossingRate")==0 || cmp_nocase_no__(var,"Barrier_Crossing_Rate")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>TVpars.back().barrierCrossingRate;
			goto nextline;
		}		
		if(cmp_nocase_no__(var,"Dispersal_Distribution")==0) {
			stringstream strstr(buf.substr(pos+1));
			if(strstr.str().length()==1) {
				strstr>>TVpars.back().mod;
			} else {
				string locstring;
				strstr>>locstring;
				if(cmp_nocase_no__(locstring,"steppingStone")==0) TVpars.back().mod='b';
				else if(cmp_nocase_no__(locstring,"Geometric")==0) TVpars.back().mod='g';
				else if(cmp_nocase_no__(locstring,"Sichel")==0) TVpars.back().mod='S';
				else if(cmp_nocase_no__(locstring,"Pareto")==0) TVpars.back().mod='P';
				
			}
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Total_Emigration_Rate")==0) {
            stringstream strstr(buf.substr(pos+1));
			strstr>>TVpars.back().Mig;
					goto nextline;
		}
		if(cmp_nocase_no__(var,"dist_max")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>TVpars.back().dist_max;
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Pareto_Shape")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>TVpars.back().Pareto_Shape;
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Geometric_Shape")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>TVpars.back().GeoG;
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Sichel_Gamma")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>TVpars.back().Sichel_Gamma;
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Sichel_Xi")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>TVpars.back().Sichel_Xi;
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Sichel_Omega")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>TVpars.back().Sichel_Omega;
			goto nextline;
		}
		if(cmp_nocase_no__(var,"ContinuousDemeSizeVariation")==0 || cmp_nocase_no__(var,"ContinuousPopSizeVariation")==0 
			|| cmp_nocase_no__(var,"ContinuousDemeSizeChange")==0 ||cmp_nocase_no__(var,"ContinuousPopSizeChange")==0 
			|| cmp_nocase_no__(var,"ContinuousVariation")==0 || cmp_nocase_no__(var,"ContinuousChange")==0) {
			string locstring;
			stringstream strstr(buf.substr(pos+1));
			strstr>>locstring;
			if(cmp_nocase_no__(locstring,"Linear")==0) {
			    TVpars.back().ContSizeChange="Linear";
			} else if(cmp_nocase_no__(locstring,"Exponential")==0) {
                TVpars.back().ContSizeChange="Exponential";
            } else if(cmp_nocase_no__(locstring,"Logistic")==0) {
                TVpars.back().ContSizeChange="Logistic";
            } else if(cmp_nocase(locstring,"False")==0 || cmp_nocase(locstring,"None")==0 || cmp_nocase(locstring,"F")==0) {
                TVpars.back().ContSizeChange="None";
            } else {cerr<<"\nBad Argument " << locstring << "  for ContinuousDemeSizeVariation in " << SettingsFilename <<  " :"<<endl;
					  cerr<<"Only Linear, Exponential, Logistic or False/None/F are implemented. I exit."<<endl;
					  if (cinGetOnError) cin.get();
					  exit(-1);
				}
			goto nextline;
		}
		if(cmp_nocase_no__(var,"LogisticGrowthRate")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>TVpars.back().LogisticGrowthRate;
			goto nextline;
		}
		if(cmp_nocase_no__(var,"Random_Translation")==0) {
			evaluateBool(random_translation,buf.substr(pos+1));
			goto nextline;
		}
        cerr<<"\n(!) Unknown keyword "<<var<<" in file "<<filename<<endl;
        if (cinGetOnError) {
            if (hacktempo) {
                cerr<<"\n Proceed to next keyword, for testing purposes ? (return/control-C) "<<endl;
                cin.get();
            }
			hacktempo=false;
        } else exit(-1);

nextline: ;	// the pleasure of sin :-)
    } while(!settings.eof());
settings.close();
return 0;
}

/***** verif options ***************************/

int apply_final_settings() {
static bool executed=false;
using namespace NS_translation;


    if (executed) {
       cerr<<"apply_final_settings() has been called twice... Program aborted. ";
       if (cinGetOnError) cin.get();
       exit(-1);
    }
    executed=true; // vraiement utile a mettre uniquement after som "new" pointer allocation in this function

    //	pauses so that the user takes notice of the typos in the settings file
    if (warningBool && cinGetOnError) {
    	cerr << "\nPress any key to resume... " << endl;
    	cin.get();
    }

//    ***********************************************************************
//    A FINAL CHECK OF LOCUS/MARKER SPECIFIC INPUT SETTINGS BEFORE SIMULATION
//    ***********************************************************************

    //	IF the total number of loci has not been specified but the mutational models have been specified
	if (locusNumVector.size() == 1 && mutModelIDVector.size() > 2) {
		locusNumVector.assign(mutModelIDVector.size(), 10);
		nMarker = locusNumVector.size() - 1;
		n_locus = nMarker * 10;
	}
	//	ELSE IF the total number of loci not specified, then use the default value stored at the zeroth index
	else if (locusNumVector.size() == 1) {
		n_locus = locusNumVector[0];
		nMarker = 1;
		locusNumVector.push_back(locusNumVector[0]);
	}
	//	ELSE sum across all the specified loci numbers to obtain the total number of loci to be simulated.
	else {
		n_locus = 0;
		nMarker = locusNumVector.size() - 1;
		for (int i = 1; i <= (int) locusNumVector.size() - 1; i++)
			n_locus += locusNumVector[i];
	}

	//	Expands unspecified/partly specified/marker-wise specified parameter values into a locus-wise specification
	if ((int) mutModelIDVector.size() != (n_locus + 1))
		expandParamVector(mutModelIDVector, "Mutation_Model");

	if ((int) mutRateVector.size() != (n_locus + 1))
		expandParamVector(mutRateVector, "Mutation_Rate");

	if ((int) minAlleleVector.size() != (n_locus + 1))
		expandParamVector(minAlleleVector, "Min_Allele_Number");

	if ((int) polyLociBoolVector.size() != (n_locus + 1))
		expandParamVector(polyLociBoolVector, "Polymorphic_Loci_Only");

	if ((int) minorAlleleFreqVector.size() != (n_locus + 1))
		expandParamVector(minorAlleleFreqVector, "Minor_Allele_Frequency");

	if ((int) varMutRateBoolVector.size() != (n_locus + 1))
		expandParamVector(varMutRateBoolVector, "Variable_Mutation_Rate");

	if ((int) kMinVector.size() != (n_locus + 1))
		expandParamVector(kMinVector, "Allelic_Lower_Bound");

	if ((int) kMaxVector.size() != (n_locus + 1))
		expandParamVector(kMaxVector, "Allelic_Upper_Bound");

	if ((int) kIniVector.size() != (n_locus + 1))
		expandParamVector(kIniVector, "Allelic_State_MRCA");

	if ((int) motifSizeVector.size() != (n_locus + 1))
		expandParamVector(motifSizeVector, "Repeated_motif_size");

	if ((int) pSMMVector.size() != (n_locus + 1))
		expandParamVector(pSMMVector, "SMM_Probability_In_TPM");

	if ((int) geomTPMVector.size() != (n_locus + 1))
		expandParamVector(geomTPMVector, "Geometric_Variance_In_TPM");

	if ((int) geomGSMVector.size() != (n_locus + 1))
		expandParamVector(geomGSMVector, "Geometric_Variance_In_GSM");

	for (int i = 1; i <= n_locus; i++) {

		//	eliminating all "minor" incompatibility issues for sequence data and/or SNPs
		if (specifyLocusSeq) {
			//	checks if the specified Equilibrium_Frequencies are all positive and sum up to 1
			if (specifyVarBaseFreqs) {
				if ((varBaseFreqs[0] < 0.0) || (varBaseFreqs[1] < 0.0) || (varBaseFreqs[2] < 0.0) || (varBaseFreqs[3] < 0.0)) {
					cerr << "\nThe values of \"Equilibrium_Frequencies\" must be positive. \nPlease re-check the file "
						 << SettingsFilename << endl << "Aborting IBDSim...Press any key to exit.";
					if (cinGetOnError)
					   cin.get();
					exit(-1);
				}
				if ((varBaseFreqs[0] + varBaseFreqs[1] + varBaseFreqs[2] + varBaseFreqs[3]) != 1.0) {
					cerr << "\nThe values of \"Equilibrium_Frequencies\" must sum up to 1. \nPlease re-check the file "
						 << SettingsFilename << endl << "Aborting IBDSim...Press any key to exit.";
					if (cinGetOnError)
					   cin.get();
					exit(-1);
				}
			}
			//	checks if Sequence_Size and MRCA_Sequence correspond when they have been user-specified
			if (defMRCASequence.size() && ((int)defMRCASequence.size() != seqSize)) {
				cerr << "\nThe specified \"Sequence_Size\" and \"MRCA_Sequence\" do not correspond. \nPlease re-check the file "
					 << SettingsFilename << endl << "Aborting IBDSim...Press any key to exit.";
				if (cinGetOnError)
				   cin.get();
				exit(-1);
			}
			//	eliminating all "minor" incompatibility issues for sequence data
			if ((mutModelIDVector[i] == "ISM") || (mutModelIDVector[i] == "SNP") || (mutModelIDVector[i] == "JC69")
					|| (mutModelIDVector[i] == "K80") || (mutModelIDVector[i] == "F81")
					|| (mutModelIDVector[i] == "HKY85") || (mutModelIDVector[i] == "TN93")) {
				if (kMinVector[i] != -1)
					kMinVector[i] = -1;
				if (kMaxVector[i] != -1)
					kMaxVector[i] = -1;
				if (kIniVector[i] != -1)
					kIniVector[i] = -1;
				if (motifSizeVector[i] != -1)
					motifSizeVector[i] = -1;
				if (pSMMVector[i] != double(-1))
					pSMMVector[i] = -1;
				if (geomTPMVector[i] != double(-1))
					geomTPMVector[i] = -1;
				if (geomGSMVector[i] != double(-1))
					geomGSMVector[i] = -1;
			}
			//	eliminating all "minor" incompatibility issues for SNPs
			if (mutModelIDVector[i] == "SNP") {
				if (mutRateVector[i] != -1)
					mutRateVector[i] = -1;
				if (!varMutRateBoolVector[i])
					varMutRateBoolVector[i] = false;
				if (minAlleleVector[i] > 2) {
					cerr << "\n\"Min_Allele_Number\" for SNPs cannot be bigger than 2. \nPlease re-check the file "
						 << SettingsFilename << endl << "Aborting IBDSim...Press any key to exit.";
					if (cinGetOnError)
					   cin.get();
					exit(-1);
			    }
			}
		}
		//	ONLY for microsatellites
		//	checks if the specified allelic bounds contain the state of the MRCA
		if ((mutModelIDVector[i] == "KAM") && (mutModelIDVector[i] == "SMM")
				&& (mutModelIDVector[i] == "TPM") && (mutModelIDVector[i] == "GSM")
				&& (kIniVector[i]) && (kIniVector[i] > kMaxVector[i] || kIniVector[i] < kMinVector[i])) {
			cerr << "\n\"Allelic_State_MRCA\" not between \"Allelic_Upper_Bound\" and \"Allelic_Lower_Bound\" at locus number "
					<< i << ". \nPlease check the file " << SettingsFilename << endl
					<< "Aborting IBDSim...Press any key to exit.";
			if (cinGetOnError)
			   cin.get();
			exit(-1);
	    }
		//	checks for 0 <= minorAlleleFreqVector < 0.5
		if ((minorAlleleFreqVector[i] < 0) && (minorAlleleFreqVector[i] > 0.5)) {
			cerr << "\n\"Minor_Allele_Frequency\" needs to be between 0 and 0.5 for locus number "
					<< i << ". \nPlease check the file " << SettingsFilename << endl
					<< "Aborting IBDSim...Press any key to exit.";
			if (cinGetOnError)
			   cin.get();
			exit(-1);
		}
		//	overrides the minAlleleVector settings if polyLociBoolVector has been set or if
		//	minorAlleleFreqVector has been specified
		if ((minAlleleVector[i] < 2)
				&& (polyLociBoolVector[i] || (minorAlleleFreqVector[i] > 0)))
			minAlleleVector[i] = 2;
	}

    if(DG2002 && n_locus > 1) {
    	printf("\nCannot write multilocus files for DG2002 (=> Set n_locus=1)");
        cerr << "Check settings. I exit." << endl;
        if (cinGetOnError)
        	cin.get();
        exit(-1);
    }

//    ***********************************************************************


    if (fichier_genepop.length()==0) fichier_genepop="gp";
    fichier_migrate=fichier_genepop+"_m";
	fichier_stepsim=fichier_genepop+"_dg";
	if (repet!=1) fichier_genepop+="_";

    if(TVpars[0].dimRes1<TVpars[0].dimRes2) {
        cerr<<"\n Found Lattice_SizeX<lattice_SizeY";
        if (TVpars.size()>1) cerr<<" in first demographic phase";
        cerr<<".\nThese two dimensions are swapped.";
        if (pauseGP) cin.get();
        int bidon=TVpars[0].dimRes1;
        TVpars[0].dimRes1=TVpars[0].dimRes2;
        TVpars[0].dimRes2=bidon;
    }


	for (vector<CTimeVaryingParams>::iterator it=TVpars.begin();it!=TVpars.end();it++)
	   if (it->dist_max<0) it->dist_max=max(it->dimRes1,it->dimRes2);


    if(cmp_nocase(boundaryType,"Circular")==0) {
                EdgeEffect="circ";
                SortieSupfnPtr1 = &sortie_sup_circ_abs1;
                SortieSupfnPtr2 = &sortie_sup_circ_abs2;
                SortieInffnPtr1 = &sortie_inf_circ_abs1;
                SortieInffnPtr2 = &sortie_inf_circ_abs2;
    } else if(cmp_nocase(boundaryType,"Absorbing")==0 || cmp_nocase(boundaryType,"Absorbing1")==0) {
                    EdgeEffect="abs";
                    SortieSupfnPtr1 = &sortie_sup_circ_abs1;
                    SortieSupfnPtr2 = &sortie_sup_circ_abs2;
                    SortieInffnPtr1 = &sortie_inf_circ_abs1;
                    SortieInffnPtr2 = &sortie_inf_circ_abs2;
    } else if(cmp_nocase(boundaryType,"Reflecting")==0) {
                        EdgeEffect="refl";
                        SortieSupfnPtr1 = &sortie_sup_refl1;
                        SortieSupfnPtr2 = &sortie_sup_refl2;
                        SortieInffnPtr1 = &sortie_inf_refl1;
                        SortieInffnPtr2 = &sortie_inf_refl2;
    } else {cout<<"\nBad Argument for Lattice_Boundaries in " << SettingsFilename <<  " :"<<endl;
					  cerr<<"Only Circular, Absorbing or Reflecting are possible options. I exit."<<endl;
				 	  if (cinGetOnError) getchar();
					  exit(-1);
    }

    if (Spec_Sample_Coord[x].size()!=Spec_Sample_Coord[y].size()) {
                cerr<<"SampleCoordinatesX and SampleCoordinatesY are of unequal size. I exit." <<endl;
			    if(cinGetOnError) cin.get();
				exit(-1);
    } else if (Spec_Sample_Coord[x].size()>0) {
        Specific_Sample_Designbool=true;
        Spec_SampleSize=Spec_Sample_Coord[x].size();
    }

    if(cmp_nocase(TVpars.back().ContSizeChange,"Linear")==0 || cmp_nocase(TVpars.back().ContSizeChange,"Exponential")==0) {
                cerr<<"Requested Linear or Exponential change in pop size in the last demographic phase. " <<endl;
                cerr<<"These changes only make sense if a further demographic phase exists. I exit." <<endl;
				TVpars.back().PrintTVpars();
			    if(cinGetOnError) cin.get();
				exit(-1);
    }

    int maxdimRes=0;
	for (vector<CTimeVaryingParams>::iterator it=TVpars.begin();it!=TVpars.end();it++) {
        maxdimRes=max(maxdimRes,it->dimRes1);
        maxdimRes=max(maxdimRes,it->dimRes2);
    }



    if( ! const_disp && (TVpars.size()==1 || maxdimRes==1)) {
        cerr<<"\nConstant_Dispersal is set to false but there is no change in time, or there is a single population:"<<endl;
        cerr<<"Constant dispersal is now set to true"<<endl;
        const_disp=true;
    }

    if(Kini!=0&&(Kini>Kmax||Kini<Kmin)) {
        cerr<<"\nProblem : Kini is not between Kmin and Kmax. Check these settings. I exit.";
        if (cinGetOnError) cin.get();
        exit(-1);
    }

    if(DG2002&&n_locus>1)
    {printf("\nCannot write multilocus files for DG2002 (=> Set n_locus=1)");
        cerr<<"Check settings. I exit."<<endl;
        if (cinGetOnError) cin.get();
        exit(-1);
    }
//if(!Specific_Sample_Designbool) {
	if(TVpars[0].dimRes2==1) vide_sampleY=1; else vide_sampleY=vide_sample;
	if(TVpars[0].dimRes1==1) vide_sampleX=1; else vide_sampleX=vide_sample;
//}
    if( ! Specific_Sample_Designbool) {
        if (xmin_sample==0) // no explicit value in settings
            xmin_sample=TVpars[0].vide;
        if (ymin_sample==0) // no explicit value in settings
            ymin_sample=TVpars[0].vide;
        if(TVpars[0].dimRes1<(dim_sample1-1)*vide_sampleX+1) {
            cerr<<"Habitat dimension LatticeSizeX< sample dimension ="<<(dim_sample1-1)*vide_sampleX+1;
            cerr<<"\n    as implied by SampleSizeX="<<dim_sample1<<" and void_sample_node="<<vide_sampleX;
            cerr<<"Check settings. I exit."<<endl;
            if (cinGetOnError) cin.get();
            exit(-1);
        }
        if(TVpars[0].dimRes2<(dim_sample2-1)*vide_sampleY+1) {
            cerr<<"Habitat dimension LatticeSizeY< sample dimension ="<<(dim_sample2-1)*vide_sampleY+1;
            cerr<<"\n    as implied by SampleSizeY="<<dim_sample2<<" and void_sample_node="<<vide_sampleY;
            cerr<<"Check settings. I exit."<<endl;
            if (cinGetOnError) cin.get();
            exit(-1);
        }
    }

    /** check that samples fit the population design when there is NO specific sampling  design:
    (i) test first coordinate
    (ii) test total size
    (iii) test steps
    **/

    if( ! Specific_Sample_Designbool && !((xmin_sample % TVpars[0].vide)==0)) {
        cerr<<"\nX Sample Coordinates are misspecified:"<<endl;
        cerr<<"The first X coordinate does not match the population design (Void_Nodes)"<<endl;
        cerr<<"Check settings. I exit."<<endl;
        if (cinGetOnError) cin.get();
        exit(-1);
    }

    if( ! Specific_Sample_Designbool && !((ymin_sample % TVpars[0].vide)==0)) {
        cerr<<"\nY Sample Coordinates are misspecified:"<<endl;
        cerr<<"The first Y coordinate does not match the population design (Void_Nodes)"<<endl;
        cerr<<"Check settings. I exit."<<endl;
        if (cinGetOnError) cin.get();
        exit(-1);
    }


    if(!Specific_Sample_Designbool) {
        if(TVpars[0].dimRes1<(xmin_sample+(dim_sample1-1)*vide_sampleX)){
            cerr<<"Habitat dimension LatticeSizeX< maximum sample coordinate ="<<xmin_sample+(dim_sample1-1)*vide_sampleX;
            cerr<<"\n    as implied by MinSampleCoordinateX="<<xmin_sample<<" ,SampleSizeX="<<dim_sample1<<" and void_sample_node="<<vide_sampleX;
            cerr<<"Check settings. I exit."<<endl;
            if (cinGetOnError) cin.get();
            exit(-1);
        }
        if(TVpars[0].dimRes2<(ymin_sample+(dim_sample2-1)*vide_sampleY)){
            cerr<<"Habitat dimension LatticeSizeY< maximum sample coordinate ="<<ymin_sample+(dim_sample2-1)*vide_sampleY;
            cerr<<"\n    as implied by MinSampleCoordinateY="<<ymin_sample<<" ,SampleSizeY="<<dim_sample2<<" and void_sample_node="<<vide_sampleY;
            cerr<<"Check settings. I exit."<<endl;
            if (cinGetOnError) cin.get();
            exit(-1);
        }
        if(xmin_sample<=0 || ymin_sample<=0){
            cerr<<"\nProblem with sample position on the lattice: X/Ymin_sample < 0";
            cerr<<"Check settings. I exit."<<endl;
            if (cinGetOnError) cin.get();
            exit(-1);
        }
    }
    if( ! Specific_Sample_Designbool && !((vide_sample % TVpars[0].vide)==0)) {
        cerr<<"\nY Sample Coordinates are misspecified:"<<endl;
        cerr<<"The spacing between samples (void_samples) is not a multiple of the spacing between occupied nodes (void_nodes)"<<endl;
        cerr<<"Check settings. I exit."<<endl;
        if (cinGetOnError) cin.get();
        exit(-1);
    }
    /** END check that samples fit the population design when there is NO specific sampling  design **/

    //RL -> RL verfi les deux tests suivants...
    if(Specific_Sample_Designbool && !((Spec_Sample_Coord[x][0] % vide_sampleX)==0)) {
        cerr<<"\nX Sample Coordinates are misspecified:"<<endl;
        cerr<<"The first X coordinate (=" << Spec_Sample_Coord[x][0] << ") does not match void_sample_node=" << vide_sampleX << ";" <<endl;
        cerr<<"Check settings. I exit."<<endl;
        if (cinGetOnError) cin.get();
        exit(-1);
    }
    if(Specific_Sample_Designbool && !((Spec_Sample_Coord[y][0] % vide_sampleY)==0)) {
        cerr<<"\nY Sample Coordinates are misspecified:"<<endl;
        cerr<<"The first Y coordinate does not match void_sample_node"<<endl;
        cerr<<"Check settings. I exit."<<endl;
        if (cinGetOnError) cin.get();
        exit(-1);
    }


    {vector<CTimeVaryingParams>::iterator it=TVpars.begin();
    int locit=1;
	while (it!=TVpars.end()) {
        
		if ((it->dimRes1 % it->vide) !=0 || (it->dimRes2 % it->vide) != 0) {
            cerr<<"\n In demographic Phase "<<locit<<" :";
            cerr<<"\n Any of lattice_size_X or lattice_size_Y is not a multiple or void_nodes. I exit";
            if (cinGetOnError) cin.get();
            exit(-1);
        }
		if(it->barrier) {
			if ((it->x1_barrier < 1) || (it->x2_barrier < 1) || (it->y1_barrier < 1) || (it->y2_barrier < 1)) {
				cerr<<"\n In demographic Phase "<<locit<<" :";
				cerr<<"\n At least one of the barrier coordinates is < 1. Check your settings. I exit."<< endl;
				if (cinGetOnError) cin.get();
				exit(-1);
			}
			if ((it->x1_barrier > it->dimRes1) || (it->x2_barrier > it->dimRes1)) {
				cerr<<"\n In demographic Phase "<<locit<<" :";
				cerr<<"\n At least one of the barrier X coordinates is larger than lattice size X. Check your settings. I exit."<< endl;
				if (cinGetOnError) cin.get();
				exit(-1);
			}
			if ((it->y1_barrier > it->dimRes2) || (it->y2_barrier > it->dimRes2)) {
				cerr<<"\n In demographic Phase "<<locit<<" :";
				cerr<<"\n At least one of the barrier Y coordinates is larger than lattice size Y. Check your settings. I exit."<< endl;
				if (cinGetOnError) cin.get();
				exit(-1);
			}
			if (it->barrierCrossingRate > 1.0) {
				cerr<<"\n In demographic Phase "<<locit<<" :";
				cerr<<"\n (!) barrierCrossingRate is > 1.0 (!) . Check your settings. I exit."<< endl;
				if (cinGetOnError) cin.get();
				exit(-1);
			}
			if ((it->y1_barrier != it->y2_barrier) && (it->x1_barrier != it->x2_barrier)) {
				cerr<<"\n In demographic Phase "<<locit<<" :";
				cerr<<"\n The barrier is nor a horizontal or vertical line. IBDSim can not consider such barriers. I exit";
				if (cinGetOnError) cin.get();
				exit(-1);
			}
			if ((it->y1_barrier == it->y2_barrier) && (it->x1_barrier > it->x2_barrier)) {
				cerr<<"\n In demographic Phase "<<locit<<" :";
				cerr<<"\n exchanging x1_barrier and x2_barrier values, because x1 > x2 in settings file."<< endl;
				int temp=it->x2_barrier;
				it->x2_barrier=it->x1_barrier;
				it->x1_barrier=temp;
				if (cinGetOnError) {
					cerr << "Press any key to resume." << endl;
					cin.get();
				}
			}
			if ((it->x1_barrier == it->x2_barrier) && (it->y1_barrier > it->y2_barrier)) {
				cerr<<"\n In demographic Phase "<<locit<<" :";
				cerr<<"\n exchanging y1_barrier and y2_barrier values, because y1 > y2 in settings file."<< endl;
				int temp=it->y2_barrier;
				it->y2_barrier=it->y1_barrier;
				it->y1_barrier=temp;
				if (cinGetOnError) {
					cerr << "Press any key to resume." << endl;
					cin.get();
				}
			}
			if (((it->x1_barrier == it->x2_barrier) && (it->y1_barrier == it->y2_barrier)) || (it->barrierCrossingRate > 0.99999)) {
				cerr<<"\n In demographic Phase "<<locit<<" :";
				cerr<<"\n Barrier settings will not have any effect because x1=x2 and y1=y2  or barrierCrossingRate > 0.99999 in settings file."<< endl;
				it->barrier=false;
				if (cinGetOnError) {
					cerr << "Press any key to resume." << endl;
					cin.get();
				}
			}
			if ( (it->barrierCrossingRate < 0.0000000001) && ( ((it->x1_barrier == 1) && (it->x2_barrier == it->dimRes1)) || ((it->y1_barrier == 1) && (it->y2_barrier == it->dimRes2)) ) ) {
				cerr<<"\n In demographic Phase "<<locit<<" :";
				cerr<<"\n (!) barrierCrossingRate is " << it->barrierCrossingRate << " (< 0.0000000001) with a complete vertical or horizontal barrier (!) ." << endl;
				cerr << " with such configuration, either IBDSim will never stop a run because there is no possible MRCA if sample is taken across the barrier" << endl;
				cerr << " or the barrier will only cut the lattice into a reacheable part and a non-reacheable one." << endl;
				cerr << " We suspect it is not the desired configuration." << endl;
				cerr << " Check your settings. I exit."<< endl;
				if (cinGetOnError) cin.get();
				exit(-1);
			}
			if ((it->x1_barrier == it->x2_barrier)) it->barrierX=true;
			if ((it->y1_barrier == it->y2_barrier)) it->barrierY=true;
		}
		
        it++;locit++;
    }
    }//end bloc local var


    if(Specific_Sample_Designbool && (Prob_Id_Matrix || calculoui || HexpNeioui || Varoui || iterativeStats || seqStatsOui)) {
        cerr<<"\n\n\nSpecific_Sample_Design=true is not compatible with"<<endl;
        cerr<<"Hexp=true,Allelic_Variance=true, Pob_id_Matrix=true or Iterative_Statistics=true "<<endl;
        cerr<<"All computations of genetic statistics will be set to false...."<<endl;
        if (cinGetOnError) {
            cout<<"Press any key to continue computations...."<<endl;
            cin.get();
        }
        calculoui=false;
        HexpNeioui=false;
        Varoui=false;
        iterativeStats=false;
        Prob_Id_Matrix=false;
        seqStatsOui=false;
	//exit(-1);
    } else {
        if((HexpNeioui ||Varoui ||iterativeStats || suiviQ || Fisoui) && !calculoui) {
            calculoui=true;
        }
		//exit(-1);
	}
	if(cmp_nocase(twoD_disp,"1DProductWithoutm0")==0) {
        vector<CTimeVaryingParams>::iterator it=TVpars.begin();
        int locit=1;
        while (it!=TVpars.end()) {
            if (it->mod != 'g' && it->mod != 'S') {
                cerr<<"\n twoD_disp==\"1DProductWithoutm0\", with incompatible dispersal model (not geometric) in demographic Phase "<<locit<<" :";
                cerr<<"\n 1DProductWithoutm0 works only with geometric dispersal. twoD_disp is set to Simple1DProduct";
                twoD_disp="Simple1DProduct";
                if (cinGetOnError) cin.get();
            }
            it++;locit++;
        }
    }

    for (unsigned int it=0;it<TVpars.size();it++) {
        dim_reseau1=max(dim_reseau1,TVpars[it].dimRes1);
        dim_reseau2=max(dim_reseau2,TVpars[it].dimRes2);
    }
    grande_dim=max(dim_reseau1,dim_reseau2);


    for (unsigned int it=0;it<TVpars.size();it++)
        if (( ! TVpars[it].zone) && (TVpars[it].vide==1)) {
            if (it>0 || !Specific_Density_Designbool) TVpars[it].fixe=true; else TVpars[it].fixe=false;
        }


    {int nonfixes=0;
        for (unsigned int it=0;it<TVpars.size();it++) if ( ! TVpars[it].fixe) nonfixes++;
        if (nonfixes>1) {
            cerr << "IBDSim is not yet designed to take into account temporal changes with different non-homogeneous density configurations." << endl;
            cerr << "Contact R. Leblois to adapt the program for such cases.\nI exit." << endl;
            if (cinGetOnError) cin.get();
            exit(-1);
        }
    }


    if(Specific_Density_Designbool) DensSpatiallyHeterog=true; else DensSpatiallyHeterog=false;
    for (unsigned int it=0;it<TVpars.size();it++) if ( ! TVpars[it].fixe) DensSpatiallyHeterog=true;
/** ancien code:
if( ! Specific_Density_Designbool && ((fixe0&&fixe1&&fixe2&&fixe3) || (fixe0&&(Gn1>2147483646)))) DensSpatiallyHeterog=false;
  else DensSpatiallyHeterog=true;

il suffit donc que l'une des conditions ne soient pas realisees pour que DensSpatiallyHeterog soit true**/

    if(DensSpatiallyHeterog){
        sym=0;
        if((!Specific_Density_Designbool) && (!TVpars[0].zone) && (!TVpars[0].fixe)) sym=TVpars[0].vide;
        for (unsigned int it=1;it<TVpars.size();it++)
            if( ( ! TVpars[it].zone ) && ( ! TVpars[it].fixe ) ) sym=max(sym,TVpars[0].vide);
        if(sym==0) sym=max(dim_reseau1,dim_reseau2);
    }
       /** ancien code
    if(DensSpatiallyHeterog){
        sym=0;
        if((!Specific_Density_Designbool) && (!zone0) && (!fixe0)) sym=vide0;
        if((!zone1)&&(!fixe1)&&(vide1>sym)&&(Gn1<2147483646)) sym=vide1;
        if((!zone2)&&(!fixe2)&&(vide2>sym)&&(Gn2<2147483646)) sym=vide2;
        if((!zone3)&&(!fixe3)&&(vide3>sym)&&(Gn3<2147483646)) sym=vide3;
        if(sym==0) sym=max(dim_reseau1,dim_reseau2);
    }
        **/
    {bool anyContSizeChange=false;
        for (unsigned int it=0;it<TVpars.size()-1;it++)
            if( cmp_nocase(TVpars[it].ContSizeChange,"None") !=0 ) anyContSizeChange=true;
        if((DensSpatiallyHeterog || Specific_Density_Designbool) && anyContSizeChange) {
            cerr << "IBDSim is not yet designed to take into account continuous deme size changes with non-homogeneous density configurations." << endl;
            cerr << "Check ContinuousDemeSizeVariation, Void_nodes and Specific_Density_Design specifications in settings";
            cerr << "I exit." << endl;
            if (cinGetOnError) cin.get();
            exit(-1);
        }
    }

    effacer_ecran();

//cout<<"fin apply_final_settings";
return 0;
}
/*****************/

